<?php

namespace Drupal\dropsolid_personalisation\Plugin\UnomiConnector;

use Dropsolid\OAuth2\Client\Http\Guzzle\GuzzleApiClientFactory;
use Dropsolid\OAuth2\Client\Provider\DropsolidPlatform;
use Drupal\Core\Form\FormStateInterface;
use Drupal\unomi\UnomiConnector\UnomiConnectorPluginBase;
use League\OAuth2\Client\Token\AccessToken;


/**
 * Dropsolid Platform Unomi connector.
 *
 * @UnomiConnector(
 *   id = "dropsolid_platform",
 *   label = @Translation("Dropsolid Platform"),
 *   description = @Translation("A Dropsolid Platform connector usable for the saas services of the Dropsolid Platform.")
 * )
 */
class DropsolidPlatformUnomiConnector extends UnomiConnectorPluginBase {

  /**
   * Drupal state.
   *
   * @var \Drupal\Core\State\StateInterface
   */
  protected $state;

  /**
   * The URL generator.
   *
   * @var \Drupal\Core\Routing\UrlGeneratorInterface
   */
  protected $urlGenerator;

  /**
   * {@inheritdoc}
   */
  public function __construct(array $configuration, $plugin_id, array $plugin_definition) {
    $configuration += $this->defaultConfiguration();
    $this->state = \Drupal::service('state');
    $this->urlGenerator = \Drupal::service('url_generator');
    parent::__construct($configuration, $plugin_id, $plugin_definition);
  }

  /**
   * {@inheritdoc}
   */
  public function defaultConfiguration() {
    return parent::defaultConfiguration() + [
        'client_id' => '',
        'client_secret' => '',
        'scheme' => 'https',
        'port' => 443,
      ];
  }

  /**
   * {@inheritdoc}
   */
  public function buildConfigurationForm(array $form, FormStateInterface $form_state) {
    $form = parent::buildConfigurationForm($form, $form_state);

    $form['auth'] = [
      '#type' => 'fieldset',
      '#title' => $this->t('Dropsolid Personalization Integration settings'),
      '#description' => $this->t(
        'To retrieve your client ID & secret: <ol>
          <li>Visit <a href="@url">Dropsolid Platform</a> and create a new keypair.</li>
          <li>Retrieve your client ID/Secret under the "OAuth2 credentials" header.</li>
          </ol>',
        [
          '@url' => 'https://platform.dropsolid.com',
        ]
      ),
      '#collapsible' => TRUE,
      '#collapsed' => empty($this->configuration['client_id']),
    ];

    $form['auth']['client_id'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Client ID'),
      '#default_value' => $this->configuration['client_id'],
      '#required' => TRUE,
    ];

    $form['auth']['client_secret'] = [
      '#type' => 'password',
      '#title' => $this->t('Client Secret'),
      '#default_value' => $this->configuration['client_secret'],
      '#description' => $this->t('If this field is left blank and the Client Secret was previously filled out, the current Client Secret will not be changed.'),
    ];

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function submitConfigurationForm(array &$form, FormStateInterface $form_state) {
    $values = $form_state->getValues();
    // Since the form is nested into another, we can't simply use #parents for
    // doing this array restructuring magic. (At least not without creating an
    // unnecessary dependency on internal implementation.)
    $values += $values['auth'];

    // For password fields, there is no default value, they're empty by default.
    // Therefore we ignore empty submissions if the user didn't change either.
    if ($values['client_secret'] === ''
      && isset($this->configuration['client_id'])
      && $values['client_id'] === $this->configuration['client_id']) {
      $values['client_secret'] = $this->configuration['client_secret'];
    }

    foreach ($values['auth'] as $key => $value) {
      $form_state->setValue($key, $value);
    }

    // Clean-up the form to avoid redundant entries in the stored configuration.
    $form_state->unsetValue('auth');

    parent::submitConfigurationForm($form, $form_state);
  }

  /**
   * {@inheritdoc}
   */
  public function viewSettings() {
    $vars = [
      '%client_id' => $this->configuration['client_id'],
      '%client_secret' => str_repeat('*', strlen($this->configuration['client_secret'])),
    ];

    $info[] = [
      'label' => $this->t('Dropsolid Platform Oauth Credentials'),
      'info' => $this->t('Client ID: %client_id | Client Secret: %client_secret', $vars),
    ];

    return $info;
  }

  /**
   * {@inheritdoc}
   * make sure this happens in the plugins. One for basic auth and one for dropsolid platform.
   */
  public function getApiClient() {
    $accessToken = $this->state->get('dropsolid_platform_access_token', '');
    if ($accessToken) {
      try {
        // Create provider.
        $provider = $this->createDropsolidPlatformProvider();

        // @todo -> Figure out why $this doesn't seem to work?

        return GuzzleApiClientFactory::createOAuth(
          $provider,
          new AccessToken($accessToken),
          [
            'callback' => [$this, 'accessToken'],
            'base_uri' => 'https://unomi-auth.live.sites.qa.dropsolid-sites.com/',
          ]
        );

      }
      catch (\Exception $e) {
        return NULL;
      }
    }
    return NULL;
  }

  /**
   * Helper function to grab the access token for the refreshMiddleware in the oauth2-dropsolid-platform library
   */
  public function accessToken(AccessToken $token) {
    $this->state->set('dropsolid_platform_access_token', $token->jsonSerialize());
  }

  /**
   * Create a new Dropsolid Platform OAuth2 Provider based on module config.
   *
   * @return DropsolidPlatform
   *   The OAuth2 Provider object
   *
   */
  protected function createDropsolidPlatformProvider() {
    return new DropsolidPlatform([
      'clientId'                => $this->configuration['client_id'],    // The client ID assigned to you by the provider
      'clientSecret'            => $this->configuration['client_secret'],    // The client password assigned to you by the provider
      'urlAuthorize'            => 'https://admin.platform.live.sites.qa.dropsolid-sites.com/oauth/authorize',
      'urlAccessToken'          => 'https://admin.platform.live.sites.qa.dropsolid-sites.com/oauth/token',
      'urlResourceOwnerDetails' => 'https://admin.platform.live.sites.qa.dropsolid-sites.com/oauth/user.info',
      'scopes' => 'cdp_admin',
    ]);
  }

}
